/*
 Standalone Temperature Logger for ATtiny85
 Version: 2.0
 Author: Alex from Inside Gadgets (www.insidegadgets.com)
 Created: 9/01/2011
 Last Modified: 10/07/2011
 
 Used to monitor temperature every x minutes/hours which is saved 
 to the built in EEPROM. You can transfer the data by connecting via 
 USB to your computer.
 
 Remember to change your fuse bits so the ATtiny85 uses 16MHz PLL, 2.7V BOD and divide clock by 8
 avrdude -p ATtiny85 -c usbtiny -U lfuse:w:0x61:m -U hfuse:w:0xdd:m -U efuse:w:0xff:m 
 
 */

//                  +-\/-+
// Ain0 (D 5) PB5  1|o   |8  Vcc
// Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1
// Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1
//            GND  4|    |5  PB0 (D 0) pwm0
//                  +----+

int thermistorPin = 3; // Analog input 3
#define ledPin PB0 // Output
#define buttonPin PB4 // Input
#define usbminusPin PB1 // USB D-

#define F_CPU 16500000

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"

volatile boolean f_pin = 0; // Pin change variable

int functionSelect = 0;
int dataCount = 1;
int delayTime = -1;

// Send the next digit from the EEPROM address
int sendnextDigit(int eepromAddress) {
	uchar digit;
	
	int value = eeprom_read_byte((uint8_t*) eepromAddress);
	
	if (value == 0) {
		return 1;
	}
	
	value -= 130; // Minus 130 from value to get real temperature
	
	// Check if value is less than 0, if so convert it to positive number and include the negative sign at the end
	boolean isNegative = false;
	if (value < 0) {
		value = value - (value * 2);
		isNegative = true;
	}
	
	nextDigit = &valueBuffer[sizeof(valueBuffer)];
	*--nextDigit = 0xff;
	*--nextDigit = 0;
	*--nextDigit = KEY_RETURN;
	
	// Break up the number and convert to USB HID values
	do{	
		digit = value % 10;
		value /= 10;
			*--nextDigit = 0;
		if (digit == 0){
			*--nextDigit = KEY_0;
		}
		else{
			*--nextDigit = KEY_1 - 1 + digit;
		}
	} while (value != 0);
	
	if (isNegative == true) {
		*--nextDigit = 0;
		*--nextDigit = KEY_MINUS;
	}
	
	return 0;
}

// Transfers the EEPROM data to USB. Stops if EEPROM value is 0 or the end of the EEPROM is reached (512).
void transferData(void) {
	boolean inusbTransfer = true;
	int readCount = 1;
	while (inusbTransfer == true) {
		usbPoll();
		if(usbInterruptIsReady() && nextDigit != NULL){ // We can send another digit
			buildReport();
			usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
			if(*++nextDigit == 0xff) { // This was terminator character
				nextDigit = NULL;
			}
		}
		if (nextDigit == NULL) {
			if (readCount < 512) {
				int checkforZero = sendnextDigit(readCount);
				if (checkforZero == true) { // End if 0
					inusbTransfer = false;
				}
				readCount++;
			}
			else {
				inusbTransfer = false;
			}
		}
	}
}

// Delay ms when we are at 1MHz by dividing by 16.5 as our F_CPU at compile time is 16.5MHz
void delay_ms_at_1mhz(int ms) {
	_delay_ms(ms / 16.5);
}

int main(void) {
	setup();
	
	DDRB |= (1<<ledPin); // Set Outputs
	DDRB &= ~((1<<buttonPin) | (1<<usbminusPin)); // Set Inputs
	
	// Pin interrupt setup
	sbi(GIMSK,PCIE); // Enable pin change interrupt
	sbi(PCMSK,PCINT4); // Apply interrupt to button pin
	
	while(1) {
		// Wait for input
		if (functionSelect == 0) {
			
			// Read delay time
			if (delayTime == -1) {
				delayTime = eeprom_read_byte((uint8_t*) 0);
				if (delayTime > 10) { // Not a normal value
					delayTime = 0;
				}
			}
			
			// Always reset variables just in case
			PORTB &= ~(1<<ledPin);
			
			system_sleep();
			
			// First test we are connected via USB by checking if the D- pin is high
			int usbConnected = PINB & (1<<usbminusPin);
			if (usbConnected) { // Send data from EEPROM to USB
				cbi(TIMSK, TOIE0); // Turn off Timer0
				PORTB |= (1<<ledPin);
				
				// Increase clock speed to 16MHz
				CLKPR = (1<<CLKPCE); // Prescaler enable
				CLKPR = 0; // Clock division factor 1 (0000)
				
				startUSB();
				transferData();
				
				// Decrease clock speed to 1MHz
				CLKPR = (1<<CLKPCE); // Prescaler enable
				CLKPR = (1<<CLKPS2); // Clock division factor 16 (0100)
				
				PORTB &= ~(1<<ledPin);
				sbi(TIMSK, TOIE0); // Turn on Timer0
			}
			else {
				// Button was pressed
				int buttonState = 0; 
				int buttonCount = 0;
				int buttonPressed = 0;
				long resetTimer = millis();
				while (millis() < resetTimer + 1500) {
					buttonState = PINB & (1<<buttonPin);
					if (buttonState) {
						buttonCount++;
						while (buttonState && (millis() < resetTimer + 1500)) {
							buttonState = PINB & (1<<buttonPin);
							buttonPressed++;
							delay_ms_at_1mhz(50);
						}
					}
				}
				
				if (buttonCount == 3) { // Start logging
					functionSelect = 1;
					dataCount = 1;
					for (int x = 0; x < 3; x++) {
						PORTB |= (1<<ledPin); // ON
						delay_ms_at_1mhz(500);
						PORTB &= ~(1<<ledPin); // OFF
						delay_ms_at_1mhz(500);
					}
				}
				else if (buttonPressed >= 20) { // Change delay time
					functionSelect = 2;
					delayTime = -1;
					PORTB |= (1<<ledPin); // ON
					delay_ms_at_1mhz(2000);
					PORTB &= ~(1<<ledPin); // OFF
				}
			}
		}
		
		// Log Temperature
		if (functionSelect == 1) {
			PORTB |= (1<<ledPin); // Turn on LED which also turns on the thermistor
			int tempValue = (int) Thermistor(analogRead(thermistorPin));
			PORTB &= ~(1<<ledPin); // OFF
			
			// Calculations to store -40C to 125C int to a single byte
			byte tempStore = tempValue + 130;
			if (tempValue <= -41) { // Temp is under min temp accepted
				tempStore = 90;
			}
			if (tempValue >= 126) { // Temp is over max temp accepted
				tempStore = 255;
			}
			
			// Write to EEPROM
			if (dataCount < 512) {
				eeprom_write_byte((uint8_t*) dataCount, tempStore);
				dataCount++;
				if (dataCount < 512) {
					eeprom_write_byte((uint8_t*) dataCount, 0); // So we know where to stop when extracting data
				}
			}
			else {
				functionSelect = 0;
			}
			
			// Calculate the delay time
			int waitCounter = 0;
			int waitTime = 0;
			f_pin = 0;
			if (delayTime == 0) {
				waitTime = 15;
			}
			if (delayTime >= 1 && delayTime < 4) {
				waitTime = (delayTime * 5) * 15;
			}
			if (delayTime >= 4 && delayTime < 6) {
				waitTime = ((delayTime-3) * 30) * 15;
			}
			if (delayTime >= 6 && delayTime < 8) {
				waitTime = ((delayTime-5) * 120) * 15;
			}
			if (delayTime == 8) {
				waitTime = 8 * 60 * 15;
			}
			if (delayTime == 9) {
				waitTime = 12 * 60 * 15;
			}
			if (delayTime == 10) {
				waitTime = 24 * 60 * 15;
			}
			
			while (waitCounter != waitTime) {
				setup_watchdog(8);
				system_sleep();
				waitCounter++;
				if (f_pin == 1) { // Did a pin change wake us up?
					functionSelect = 0;
					
					// Disable the watchdog
					MCUSR &= ~(1<<WDRF);
					WDTCR |= (1<<WDCE) | (1<<WDE);
					WDTCR = 0;
				  
					waitCounter = waitTime; // Break out of the loop
					for (int x = 0; x < 2; x++) {
						PORTB |= (1<<ledPin); // ON
						delay_ms_at_1mhz(500);
						PORTB &= ~(1<<ledPin); // OFF
						delay_ms_at_1mhz(500);
					}
				}
			}
		}
		
		// Configure the delay time
		if (functionSelect == 2) {
			system_sleep();
			
			int buttonState = PINB & (1<<buttonPin);
			if (buttonState) {
				delay_ms_at_1mhz(500);
				int buttonPressed = false;
				buttonState = PINB & (1<<buttonPin);
				if (buttonState && delayTime >= 0) {
					delay_ms_at_1mhz(1500);
					buttonState = PINB & (1<<buttonPin);
					if (buttonState) {
						functionSelect = 0;
						eeprom_write_byte((uint8_t*)0, delayTime);
						for (int x = 0; x < 3; x++) {
							PORTB |= (1<<ledPin); // ON
							delay_ms_at_1mhz(500);
							PORTB &= ~(1<<ledPin); // OFF
							delay_ms_at_1mhz(500);
						}
					}
					else {
						buttonPressed = true;
					}
				}
				if (buttonState == false || buttonPressed == true) {
					delayTime++;
					PORTB |= (1<<ledPin); // ON
					delay_ms_at_1mhz(500);
					PORTB &= ~(1<<ledPin); // OFF
					
					if (delayTime > 10) { // Too many presses
						delayTime = -1;
						PORTB |= (1<<ledPin); // ON
						delay_ms_at_1mhz(1500);
						PORTB &= ~(1<<ledPin); // OFF
					}
				}
			}
		}
	}
	return 0;
}

ISR(WDT_vect) {
}

ISR(PCINT0_vect) {
	f_pin = 1; // Set global flag
}
